#!/usr/bin/python

"""Removes unnecessary info that is added from NetLogo in multiple files

Required libraries:
-os
-pandas
-sys

"""

import os
import pandas as pd
import sys
from os import listdir

# Input
root_folder = os.getcwd()
global input_folder
global output_folder
input_folder = root_folder + "\\input\\"
output_folder = root_folder + "\\output\\"
column_titles = ["[run number]",
                 "seed",
                 "sum-of-target-area-hominins",
                 "sum-of-founder-population-size-reached",
                 "crossing-success-rate-individuals",
                 "crossing-success-rate-founder-population",
                 "death-dehydration",
                 "death-exhaustion",
                 "death-hypothermia",
                 "death-left_map_land",
                 "death-left_map_water",
                 "death-random",
                 "sum-of-deaths",
                 "attempts-source-water",
                 "mean-individual-hours-water-crossing"]
sort_col = "[run number]"

# Subprocedures
def define_workspace(input_folder, output_folder):
    has_input_folder = False # intial value
    
    if os.path.isdir(output_folder) == False:
        os.mkdir(output_folder)

    if os.listdir(output_folder) != []:
        print ("Output folder must be empty!")
        sys.exit()
    
    if os.path.isdir(input_folder) == True:
        has_input_folder = True
        
    else:
        has_input_folder = False
        print ("No input folder!")
    
    if has_input_folder == True:
        print ("Input folder: " + input_folder)
        
    print ("Output folder: " + output_folder)

def delete_first_six_lines_of_input(
        input_file, output_file):
            
    f = open(input_file, 'r')
    lines = f.readlines()
    f.close()
    
    f = open(output_file, 'w')
    del lines[0:6]
    f.writelines(lines)
    f.close()
    return output_file

def remove_quotation_marks_in_file(file):
    f = open(file)
    r = f.read().replace('"','')
    f.close()
    f = open(file, "w")
    f.write(r)
    f.close()
    return file

def remove_collumns_in_csv(csv_file, keep_collumns):
    f = pd.read_csv(csv_file)
    new_f = f[keep_collumns]
    new_f = new_f.sort_values(by=sort_col)
    new_f.to_csv(csv_file, index=False)
    return csv_file

def remove_unnecessary_information_from_netlogo_output(input_csv, output_csv_name):
    file_without_first_six_lines = delete_first_six_lines_of_input(input_csv, output_csv_name)
    file_without_quotation_marks = remove_quotation_marks_in_file(file_without_first_six_lines)
    print (output_csv_name + " created.")
    return output_csv_name

# Execution

define_workspace(input_folder, output_folder)
for file in listdir(input_folder):
    output_csv_name = str.replace(file, ".", "_") + "_out.csv"
    file = input_folder + file
    output_csv_name = output_folder + output_csv_name
    output_file = remove_unnecessary_information_from_netlogo_output(file, output_csv_name)
    remove_collumns_in_csv(output_file, column_titles)

