"""Calculates factorial statistics for two variables

"""

# 1. Summarize 100 runs per setting as mean value
#
#
# Results table template
# pr/dr_|_10_|_15_|_20_|_25_|_30_|
# __0___|____|____|____|____|____|
# __0.1_|____|____|____|____|____|
# __0.2_|____|____|____|____|____|
# __0.3_|____|____|____|____|____|
# __0.4_|____|____|____|____|____|

import numpy as np
import matplotlib.lines as lines
import matplotlib.pyplot as plt
from  matplotlib.colors import LinearSegmentedColormap
import os
import pandas as pd

# Input dictionary ->
# csv-file :
# [graph_title, parameter1, parameter1-values, parameter1-unit]
input_dict = {
    'TwoFactorSensitivity_ScenarioA_water-barrier-size_vs_perception-radius.csv' :
    ['Scenario A',
     'perception-radius'  , [10, 15, 20, 25, 30], '[km]',
     'water-barrier-size', [5, 10, 15, 20, 25], '[km]'],
    'TwoFactorSensitivity_ScenarioB_water-barrier-size_vs_perception-radius.csv' :
    ['Scenario B',
     'perception-radius'  , [10, 15, 20, 25, 30], '[km]',
     'water-barrier-size', [5, 10, 15, 20, 25], '[km]'],
    'TwoFactorSensitivity_ScenarioC_water-barrier-size_vs_perception-radius.csv' :
    ['Scenario C',
     'perception-radius'  , [10, 15, 20, 25, 30], '[km]',
     'water-barrier-size', [5, 10, 15, 20, 25], '[km]'],
    'TwoFactorSensitivity_ScenarioD_water-barrier-size_vs_perception-radius.csv' :
    ['Scenario D',
     'perception-radius'  , [10, 15, 20, 25, 30], '[km]',
     'water-barrier-size', [5, 10, 15, 20, 25], '[km]']
    }
output_variable = 'crossing-success-rate-founder-population'

# Procedures
def calc_stats_and_make_2D_raster(input_csv,
                                  graph_title,
                                  parameter1, pv1, parameter1_unit,
                                  parameter2, pv2, parameter2_unit,
                                  output_variable):
    # Load dataframe
    df = pd.read_csv(input_csv)

    # Define conditions column 1
    con00 = (df[parameter1] == pv1[0]) & (df[parameter2] == pv2[0])
    df_con00 = df[con00]
    mean00 = df_con00[output_variable].mean()

    con01 = (df[parameter1] == pv1[0]) & (df[parameter2] == pv2[1])
    df_con01 = df[con01]
    mean01 = df_con01[output_variable].mean()

    con02 = (df[parameter1] == pv1[0]) & (df[parameter2] == pv2[2])
    df_con02 = df[con02]
    mean02 = df_con02[output_variable].mean()

    con03 = (df[parameter1] == pv1[0]) & (df[parameter2] == pv2[3])
    df_con03 = df[con03]
    mean03 = df_con03[output_variable].mean()

    con04 = (df[parameter1] == pv1[0]) & (df[parameter2] == pv2[4])
    df_con04 = df[con04]
    mean04 = df_con04[output_variable].mean()

    if len(pv1) == 6 & len(pv2) == 6:
        con05 = (df[parameter1] == pv1[0]) & (df[parameter2] == pv2[5])
        df_con05 = df[con05]
        mean05 = df_con05[output_variable].mean()
        
    # Define conditions column 2
    con10 = (df[parameter1] == pv1[1]) & (df[parameter2] == pv2[0])
    df_con10 = df[con10]
    mean10 = df_con10[output_variable].mean()

    con11 = (df[parameter1] == pv1[1]) & (df[parameter2] == pv2[1])
    df_con11 = df[con11]
    mean11 = df_con11[output_variable].mean()

    con12 = (df[parameter1] == pv1[1]) & (df[parameter2] == pv2[2])
    df_con12 = df[con12]
    mean12 = df_con12[output_variable].mean()

    con13 = (df[parameter1] == pv1[1]) & (df[parameter2] == pv2[3])
    df_con13 = df[con13]
    mean13 = df_con13[output_variable].mean()

    con14 = (df[parameter1] == pv1[1]) & (df[parameter2] == pv2[4])
    df_con14 = df[con14]
    mean14 = df_con14[output_variable].mean()

    if len(pv1) == 6 & len(pv2) == 6:
        con15 = (df[parameter1] == pv1[1]) & (df[parameter2] == pv2[5])
        df_con15 = df[con15]
        mean15 = df_con15[output_variable].mean()

    # Define conditions column 3
    con20 = (df[parameter1] == pv1[2]) & (df[parameter2] == pv2[0])
    df_con20 = df[con20]
    mean20 = df_con20[output_variable].mean()

    con21 = (df[parameter1] == pv1[2]) & (df[parameter2] == pv2[1])
    df_con21 = df[con21]
    mean21 = df_con21[output_variable].mean()

    con22 = (df[parameter1] == pv1[2]) & (df[parameter2] == pv2[2])
    df_con22 = df[con22]
    mean22 = df_con22[output_variable].mean()

    con23 = (df[parameter1] == pv1[2]) & (df[parameter2] == pv2[3])
    df_con23 = df[con23]
    mean23 = df_con23[output_variable].mean()

    con24 = (df[parameter1] == pv1[2]) & (df[parameter2] == pv2[4])
    df_con24 = df[con24]
    mean24 = df_con24[output_variable].mean()

    if len(pv1) == 6 & len(pv2) == 6:
        con25 = (df[parameter1] == pv1[2]) & (df[parameter2] == pv2[5])
        df_con25 = df[con25]
        mean25 = df_con25[output_variable].mean()

    # Define conditions column 4
    con30 = (df[parameter1] == pv1[3]) & (df[parameter2] == pv2[0])
    df_con30 = df[con30]
    mean30 = df_con30[output_variable].mean()

    con31 = (df[parameter1] == pv1[3]) & (df[parameter2] == pv2[1])
    df_con31 = df[con31]
    mean31 = df_con31[output_variable].mean()

    con32 = (df[parameter1] == pv1[3]) & (df[parameter2] == pv2[2])
    df_con32 = df[con32]
    mean32 = df_con32[output_variable].mean()

    con33 = (df[parameter1] == pv1[3]) & (df[parameter2] == pv2[3])
    df_con33 = df[con33]
    mean33 = df_con33[output_variable].mean()

    con34 = (df[parameter1] == pv1[3]) & (df[parameter2] == pv2[4])
    df_con34 = df[con34]
    mean34 = df_con34[output_variable].mean()

    if len(pv1) == 6 & len(pv2) == 6:
        con35 = (df[parameter1] == pv1[3]) & (df[parameter2] == pv2[5])
        df_con35 = df[con35]
        mean35 = df_con35[output_variable].mean()

    # Define conditions column 5
    con40 = (df[parameter1] == pv1[4]) & (df[parameter2] == pv2[0])
    df_con40 = df[con40]
    mean40 = df_con40[output_variable].mean()

    con41 = (df[parameter1] == pv1[4]) & (df[parameter2] == pv2[1])
    df_con41 = df[con41]
    mean41 = df_con41[output_variable].mean()

    con42 = (df[parameter1] == pv1[4]) & (df[parameter2] == pv2[2])
    df_con42 = df[con42]
    mean42 = df_con42[output_variable].mean()

    con43 = (df[parameter1] == pv1[4]) & (df[parameter2] == pv2[3])
    df_con43 = df[con43]
    mean43 = df_con43[output_variable].mean()

    con44 = (df[parameter1] == pv1[4]) & (df[parameter2] == pv2[4])
    df_con44 = df[con44]
    mean44 = df_con44[output_variable].mean()

    if len(pv1) == 6 & len(pv2) == 6:
        con45 = (df[parameter1] == pv1[4]) & (df[parameter2] == pv2[5])
        df_con45 = df[con45]
        mean45 = df_con45[output_variable].mean()

    if len(pv1) == 6 & len(pv2) == 6:
        # Define conditions column 6
        con50 = (df[parameter1] == pv1[5]) & (df[parameter2] == pv2[0])
        df_con50 = df[con50]
        mean50 = df_con50[output_variable].mean()

        con51 = (df[parameter1] == pv1[5]) & (df[parameter2] == pv2[1])
        df_con51 = df[con51]
        mean51 = df_con51[output_variable].mean()

        con52 = (df[parameter1] == pv1[5]) & (df[parameter2] == pv2[2])
        df_con52 = df[con52]
        mean52 = df_con52[output_variable].mean()

        con53 = (df[parameter1] == pv1[5]) & (df[parameter2] == pv2[3])
        df_con53 = df[con53]
        mean53 = df_con53[output_variable].mean()

        con54 = (df[parameter1] == pv1[5]) & (df[parameter2] == pv2[4])
        df_con54 = df[con54]
        mean54 = df_con54[output_variable].mean()

        if len(pv1) == 6 & len(pv2) == 6:
            con55 = (df[parameter1] == pv1[5]) & (df[parameter2] == pv2[5])
            df_con55 = df[con55]
            mean55 = df_con45[output_variable].mean()

    # Results dataframe
    print('Create results dataframe (MIRRORED TO FIGURE!):')
    if len(pv1) == 5 & len(pv2) == 5:
        results_dict = {
            pv1[0] : [mean00, mean01, mean02, mean03, mean04],
            pv1[1] : [mean10, mean11, mean12, mean13, mean14],
            pv1[2] : [mean20, mean21, mean22, mean23, mean24],
            pv1[3] : [mean30, mean31, mean32, mean33, mean34],
            pv1[4] : [mean40, mean41, mean42, mean43, mean44]
            }    

    results_df = pd.DataFrame(results_dict, index=pv2)
    print(str(results_df) + '\n')
    print('Median of results dataframe: ' + str(results_df.median().median()))
    results_csv = os.path.splitext(input_csv)[0] + '_means.csv'
    results_df.to_csv(results_csv)
    print(results_csv + ' created.')

    print ('Create results array (SAME LIKE FIGURE!)')
    if len(pv1) == 5 & len(pv2) == 5:
        results_array = np.array([
            [mean04, mean14, mean24, mean34, mean44],
            [mean03, mean13, mean23, mean33, mean43],
            [mean02, mean12, mean22, mean32, mean42],
            [mean01, mean11, mean21, mean31, mean41],
            [mean00, mean10, mean20, mean30, mean40]
            ])
    print (str(results_array))    
    
    # Graph
    xticks = np.arange(len(pv2))
    yticks = np.arange(len(pv1))
    pv2.reverse() 
    fig, ax = plt.subplots(figsize=(5,5))

    ax.set_title(graph_title)
    ax.set_xlabel(parameter1 + ' ' + parameter1_unit)
    ax.set_ylabel(parameter2 + ' ' + parameter2_unit)
    extent = (0, results_array.shape[1], results_array.shape[0], 0)
    im = ax.imshow(results_array,
                   cmap=LinearSegmentedColormap.from_list('rg',["r", "w", "g"], N=256),
                   extent=extent)
    cbar = fig.colorbar(im, ax=ax, fraction=0.046, pad=0.04) # fraction=0.046, pad=0.0 shapes the colorbar in the size                                                  # of the plot, but I don't know why.
    cbar_label = output_variable
    ax.grid(color='black', linestyle='-', linewidth=2)
    line = lines.Line2D([0, 5], [5, 0],
                        lw=1, color='black', axes=ax) # diagonal line
    ax.add_line(line)
    if graph_title == 'Scenario A' or graph_title == 'Scenario B':
        ax.scatter(2.5,2.5, c='yellow', marker='*', s=200) # mark the median
        ax.scatter(2.5,3.5, c='blue', marker='*', s=200) # mark the default
    if graph_title == 'Scenario C':
        ax.scatter(2.5,2.5, c='yellow', marker='*', s=200) # mark the median
        ax.scatter(2.5,3.5, c='blue', marker='*', s=200) # mark the default
    if graph_title == 'Scenario D':
        ax.scatter(2.5,2.5, c='yellow', marker='*', s=200) # mark the median
        ax.scatter(2.5,3.5, c='blue', marker='*', s=200) # mark the default
    if output_variable == 'crossing-success-rate-founder-population':
        cbar_label = 'CSR'
    cbar.set_label(cbar_label, rotation=270)
    plt.xticks(xticks, pv1)
    plt.yticks(yticks, pv2)
    ax.set_xticklabels('') # hide major tick labels
    ax.set_xticks([0.5, 1.5, 2.5, 3.5, 4.5], minor=True) # position of minor ticks
    ax.set_xticklabels(pv1, minor=True) # minor x tick labels
    ax.set_yticklabels('') # hide major tick labels
    ax.set_yticks([0.5, 1.5, 2.5, 3.5, 4.5], minor=True) # position of minor ticks
    ax.set_yticklabels(pv2, minor=True) # minor y tick labels
    if graph_title == 'Scenario A' or graph_title == 'Scenario B':
        im.set_clim(0,1) # set limits for colobar
    if graph_title == 'Scenario C' or graph_title == 'Scenario D':
        im.set_clim(0,1) # set limits for colobar
    output_png = os.path.splitext(input_csv)[0] + '.png'
    plt.savefig(output_png)
    print(output_png + ' created. \n')

# Execution
for input_csv in input_dict:
    graph_title     = input_dict[input_csv][0]
    parameter1      = input_dict[input_csv][1]
    pv1             = input_dict[input_csv][2]
    parameter1_unit = input_dict[input_csv][3]  
    parameter2      = input_dict[input_csv][4]
    pv2             = input_dict[input_csv][5]
    parameter2_unit = input_dict[input_csv][6]
    print('Calculate 2D-Raster for ' + input_csv)
    print('parameter1: ' + parameter1)
    print('parameter1-values: ' + str(pv1))
    print('parameter2: ' + parameter2)
    print('parameter2-values: ' + str(pv2))
    calc_stats_and_make_2D_raster(input_csv,
                                  graph_title,
                                  parameter1, pv1, parameter1_unit,
                                  parameter2, pv2, parameter2_unit,
                                  output_variable)
    
    
    
